# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class TemplateMediation(Template):
    'Uses the Stata command <a href="http://www.stata-journal.com/article.html?article=st0238" target="blank">gformula</a> to estimate causal effects allowing for mediating variables - developed for use with eBook.'

    __version__ = '1.0.0'

    tags = [ 'Model', 'Causal' ]
    engines = ['Stata_script']

#exposure & mediator variables: more than one (i.e. DataMatrix) allowed or not (i.e. DataVector)?
#post_confs - OK for this example, but if >1 variable see note on ordering in Stata article
    inputs = '''
x = DataMatrix('Select all variables to be modelled')
y = DataVector('Outcome variable: ')
numeqns = Integer('How many equations would you like to specify?', help="Equations should be specified for the outcome variable, time-varying confounders, and the time-varying exposure (needed for simulation under the observational regime). If there is censoring due to death, then the command used for the model for death should also be specified. The equation for any particular variable (e.g. a time-varying confounder, L) must be the same for each visit: i.e. please specify only one equation for such variables.")
for i in range(0, int(numeqns)):
    context['dependeqn' + str(i+1)] = DataVector('Dependent variable for equation ' + str(i+1) )
    context['command' + str(i+1)] = Text('Which command would you like to use to model this variable?', ['regress', 'logit', 'mlogit', 'ologit'], help="For a time-to-event outcome (including the variable representing death, if there's censoring due to death), logit should be the chosen command because the time to the event (or death) is given (and simulated) as a sequence of binary variables.")
    context['independeqn' + str(i+1)] = DataMatrix('Independent variable(s) for equation ' + str(i+1) )
for i in range(0, int(numeqns)):
    if context['independeqn'+str(i+1)]:
        for var in context['independeqn'+str(i+1)]:
            context[var + 'independeqn' + str(i+1) + '_cat'] = Boolean('Is ' + var + ' categorical in equation ' + str(i+1))
derivedYN = Boolean('Do you want to specify any derived variables (e.g. interactions)?', help="Lagged variables themselves should not be included here, but variables derived using one or more lagged variables should be included. The derived variables must exist in the original dataset (i.e. the list of variables you specified initially). Only dervied variables based on the time-varying confounders need to be specified, because these need to be simulated. Interactions, say, between time-fixed confounders can simply be included themselves in the list of time-fixed confouders.")
if derivedYN:
    derivedvar = DataMatrix('Which variables do you wish to specify as derived?: ')
    for var in derivedvar:
        context[var + '_derrules'] = Text('Provide syntax indicating how ' + var + ' is derived (e.g. YourVariableA*YourVariableB)')
exposevar = DataMatrix('What are the exposure variables?')
for var in exposevar:
    context[var + '_baseline'] = Integer('What value of ' + var + ' would you like to be taken as the baseline value?')
mediatorvar = DataMatrix('What are the mediator variables?')
controlYN = Boolean('Do you want to control the mediators when estimating the CDE? (If not, only NDEs & NIEs are estimated)')
if controlYN:
    for var in mediatorvar:
        context[var + '_control'] = Integer('At which value do you wish ' + var + ' to be controlled?')
base_confsvar = DataMatrix('What are the confounders of the exposure-outcome relationship (and exposure-mediator relationship for estimating NDE and NIE)?')
post_confsvar = DataMatrix('What are the confounders of the mediator-outcome relationships affected by the exposure?')
readytogo = Boolean('Do you want to fit the model now?')
'''

#I need someway to list all the variables I use in mainvarlist - I will have them all above, but some names may repeat, so returning a list of unique names...
#...compiled from a number of different sources would be good if poss. For now, as a workaround, I have 'x', but obviously a bit clumsy!

#deleted this from second line of below: ssc install gformula, replace
#but ultimately needs to be added again, but with change which means only downloads if not installed (or perhaps checks for more recent version)

#for London? cd ${directory} 

    statascript = '''
% if readytogo:
capture which gformula
if _rc ssc install gformula
gformula \\
% for p in x:
 ${p}\\
% endfor
, mediation outcome(${y}) \\
commands(\\
% for j in range(0, int(numeqns)):
% if context['dependeqn'+str(j+1)]:
${context['dependeqn'+str(j+1)]}:\\
${context['command'+str(j+1)]}\\
% if j+1 < int(numeqns):
, \\
% endif
% endif
% endfor
) equations(\\
% for j in range(0, int(numeqns)):
% if context['dependeqn'+str(j+1)]:
${context['dependeqn'+str(j+1)]}:\\
% for p in context['independeqn'+str(j+1)]:
% if context[p + 'independeqn' + str(j+1) + '_cat']:
 i.${p}\\
% else:
 ${p}\\
% endif 
% endfor
% if j+1 < int(numeqns):
, \\
% endif
% endif
% endfor
) \\
% if derivedYN:
derived(\\
% for p in derivedvar:
${p} \\
% endfor
) \\
derrules(\\
% for p in derivedvar:
${p}:\\
% if context[p + '_derrules']:
${context[p + '_derrules']} \\
% endif
% endfor
) \\
% endif
exposure(\\
% for p in exposevar:
${p} \\
% endfor
) \\
baseline(\\
% for p in exposevar:
${p}:\\
% if context[p + '_baseline']:
${context[p + '_baseline']} \\
% endif
% endfor
) mediator(\\
% for p in mediatorvar:
${p} \\
% endfor
) \\
% if controlYN:
control(\\
% for p in mediatorvar:
${p}:\\
% if context[p + '_control']:
${context[p + '_control']} \\
% endif
% endfor
) \\
% endif
base_confs(\\
% for p in base_confsvar:
${p} \\
% endfor
) post_confs(\\
% for p in post_confsvar:
${p} \\
% endfor
) \\
% endif
sam(50)
'''
